# -*- coding: utf-8 -*-

import inspect
import test
import sys

try:
    # For Python 3:
    import builtins
except:
    # For Python 2:
    import __builtin__ as builtins
    
builtin_module = "__builtin__" if sys.version_info.major == 2 else "builtins"


# For more information see:
# https://kb.froglogic.com/squish/howto/using-squish-functions-python-modules-packages/

def import_squish_symbols_decorator(to_be_decorated):
    
    def wrapper(*args, **kwargs):
        func = __add_automatic_import_of_squish_functions(to_be_decorated)
        import_into_module = sys.modules[to_be_decorated.__module__]
        try:
            return func(*args, **kwargs)
        finally:
            # Needs to be called after we return - otherwise we get undefined symbols
            __clean_imports(import_into_module)
    return wrapper


def import_squish_symbols():
    # For regular use like a normal function:
    # https://stackoverflow.com/questions/40652688/how-to-access-globals-of-parent-module-into-a-sub-module
    __import_squish(sys.modules[dict(inspect.getmembers(inspect.stack()[1][0]))["f_globals"]["__name__"]])


def __import_squish(import_into_module):
    # Do the equivalent of "from squish import *":
    for n in dir(import_into_module):
        x = getattr(import_into_module, n)
        # callable() returns True for classes as well,
        # but we do not want wrap classes, so exclude
        # them by checking via inspect.isclass()
        # explicitly:
        if callable(x) and not inspect.isclass(x):
            setattr(import_into_module, n, __add_automatic_import_of_squish_functions(x))

    __import_regular_squish_modules(import_into_module)


def __add_automatic_import_of_squish_functions(func):
    def wrapper(*args, **kwargs):
        import_into_module = sys.modules[func.__module__]

        # https://stackoverflow.com/questions/1676835/how-to-get-a-reference-to-a-module-inside-the-module-itself/1676860#1676860
        import_into_module.__dict__.update(sys.modules["squish"].__dict__)

        # Restore built-ins - this restores behavior as it
        # is when not using this module:
        import_into_module.__dict__.update(sys.modules[builtin_module].__dict__)

        __import_regular_squish_modules(import_into_module)

        return func(*args, **kwargs)
    return wrapper

def __import_regular_squish_modules(import_into_module):
    for x in ["test", "testData", "object", "objectMap", "squishinfo"]:
        setattr(import_into_module, x, sys.modules[x])    
        
def __clean_imports(import_into_module):
    # remove the imports again
    for x in ["test", "testData", "object", "objectMap", "squishinfo"]:
        if hasattr(import_into_module, x):
            delattr(import_into_module, x)
    # Restore Python object definition
    setattr(import_into_module, 'object', builtins.object)
